-----------------------------------
-- Setting up scope and libs
-----------------------------------

local AddonName = "iGear"
iGear = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0");

local QTip = LibStub("LibQTip-1.0");
local Crayon = LibStub("LibCrayon-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

-----------------------------------------
-- Variables, functions and colors
-----------------------------------------

local Tooltip;
local RepairCosts = 0;

local COLOR_GOLD = "|cfffed100%s|r";

-- general slots
local SLOT_NAME, SLOT_ID, SLOT_IS_EQUIPPED, SLOT_COLOR, SLOT_ITEM_NAME = 1, 2, 9, 10, 11;
-- durability
local SLOT_CAN_DURABILITY, SLOT_DURABILITY, SLOT_REPAIR_COST = 3, 4, 6;
-- gems
local SLOT_GEMS_EMPTY = 5;
-- enchants
local SLOT_CAN_ENCHANT, SLOT_ENCHANT = 7, 8;

iGear.EquipSlots = {
	{"HeadSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"NeckSlot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"ShoulderSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"BackSlot", 0, false, 0, 0, 0, true, 0, false, "", ""},
	{"ChestSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"WristSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"HandsSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"WaistSlot", 0, true, 0, 0, 0, false, 0, false, "", ""},
	{"LegsSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"FeetSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"Finger0Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"Finger1Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"Trinket0Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"Trinket1Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"MainHandSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"SecondaryHandSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"RangedSlot", 0, false, 0, 0, 0, false, 0, false, "", ""}
};
local EquipSlots = iGear.EquipSlots;

local function tclear(t, wipe)
	if( type(t) ~= "table" ) then return end;
	for k in pairs(t) do
		t[k] = nil;
	end
	t[''] = 1;
	t[''] = nil;
	if( wipe ) then
		t = nil;
	end
end

--------------------------------------------------------------------
-- Initialize static table changes according to players class
--------------------------------------------------------------------

-- TO BE REMOVED WITH TOC 50000 AND ABOVE!
--=========================================
-- There are some differents between the classes. Hunter bows (ranged slot), can be enchanted.
-- Paladin Relics (ranged slot, too), canNOT be enchanted. So, when I play a Hunter, ranged SLOT_CAN_ENCHANT must be true.
-- Let's do this job in a separate scope!
do
	local _, class = UnitClass("player");
	for _, slot in ipairs(EquipSlots) do
		
		-- WARRIOR can wear bows
		-- ROGUE can wear bows
		-- HUNTER can wear bows
		if( slot[SLOT_NAME] == "RangedSlot" ) then
			if( class == "WARRIOR" or class == "ROGUE" or class == "HUNTER" ) then
				slot[SLOT_CAN_ENCHANT] = true;
			end
		end
		
	end
end

-----------------------------
-- Setting up the feed
-----------------------------

local function Feed_OnEnter(anchor)
	-- when there are no problems, we don't need the tooltip.
	if( iGear:GetNumProblems(1) == 0 ) then
		return;
	end

	-- force other i-Series qtip's to hide, when this tip is to be shown
	for k, v in QTip:IterateTooltips() do
		if( type(k) == "string" and strsub(k, 1, 6) == "iSuite" ) then
			v:Release(k);
		end
	end
	
	Tooltip = QTip:Acquire("iSuite"..AddonName);
	Tooltip:SetAutoHideDelay(0.1, anchor);
	Tooltip:SmartAnchorTo(anchor);
	iGear:UpdateTooltip();
	Tooltip:Show();
end

iGear.Feed = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",
	icon = "Interface\\Minimap\\Tracking\\Repair",
	OnEnter = Feed_OnEnter,
});

----------------------
-- OnInitialize
----------------------

function iGear:OnInitialize()
	for _, slot in ipairs(EquipSlots) do
		slot[SLOT_ID] = GetInventorySlotInfo(slot[SLOT_NAME]);
	end
	
	self:RegisterEvent("PLAYER_DEAD", "DurabilityChange");
	self:RegisterEvent("PLAYER_UNGHOST", "DurabilityChange");
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "DurabilityChange");
	self:RegisterEvent("UPDATE_INVENTORY_DURABILITY", "DurabilityChange");
end

-----------------------
-- Num problems
-----------------------

function iGear:GetProblem(s, repair)
	if( not s[SLOT_IS_EQUIPPED] ) then
		return false;
	end
	
	if( repair and s[SLOT_CAN_DURABILITY] and s[SLOT_DURABILITY] ~= 100 ) then
		return true;
	elseif( s[SLOT_GEMS_EMPTY] > 0 ) then
		return true;
	elseif( s[SLOT_CAN_ENCHANT] and s[SLOT_ENCHANT] == 0 ) then
		return true;
	end
	
	return false;
end

function iGear:GetNumProblems(repair)
	local problems = 0;
	
	for _, s in ipairs(EquipSlots) do
		if( self:GetProblem(s, repair) ) then
			problems = problems + 1;
		end
	end
	
	return problems;
end

--------------------------
-- Get Item Infos
--------------------------

CreateFrame("GameTooltip", "iGearScanTip", UIParent, "GameTooltipTemplate");

local NumMissingGems;
do
	local scanForTex = {
		"Interface\\ItemSocketingFrame\\UI-EmptySocket-Meta",
		"Interface\\ItemSocketingFrame\\UI-EmptySocket-Red",
		"Interface\\ItemSocketingFrame\\UI-EmptySocket-Yellow",
		"Interface\\ItemSocketingFrame\\UI-EmptySocket-Blue",
		"Interface\\ItemSocketingFrame\\UI-EmptySocket-CogWheel",
		"Interface\\ItemSocketingFrame\\UI-EmptySocket-Hydraulic",
		"Interface\\ItemSocketingFrame\\UI-EmptySocket-Prismatic",
		"Interface\\ItemSocketingFrame\\UI-EmptySocket"
	};
	
	NumMissingGems = function(slotNum)
		local missing = 0
		
		for i = 1, 4 do
			if ( _G["iGearScanTipTexture"..i] ) then
				_G["iGearScanTipTexture"..i]:SetTexture("");
			end
		end

		iGearScanTip:ClearLines();
		iGearScanTip:SetInventoryItem("player", slotNum);
		
		local tex;
		for i = 1, 4 do
			tex = _G["iGearScanTipTexture"..i]:GetTexture();
			
			if( tex ) then
				for j = 1, #scanForTex do
					if( tex == scanForTex[j] ) then
						missing = missing + 1;
					end
				end
			end
		end
		
		iGearScanTip:Hide();
		return missing;
	end
end

local function GetItemEquippedAndCost(slotNum)
	iGearScanTip:ClearLines();
	
	local equipped, _, cost = iGearScanTip:SetInventoryItem("player", slotNum);
	
	iGearScanTip:Hide();
	return equipped, cost;
end

local function GetItemDurability(s)
	if( not s[SLOT_CAN_DURABILITY] ) then
		return;
	end
	
	if( GetInventoryItemBroken("player", s[SLOT_ID]) ) then
		return 0;
	end
	local current, maximum = GetInventoryItemDurability(s[SLOT_ID]);
	
	if( not current or not maximum ) then
		return;
	end
	local durability = 100 * current / maximum;
	
	if( durability - abs(durability) >= 0.5 ) then
		durability = ceil(durability);
	else
		durability = floor(durability);
	end
	
	return durability;
end

local function GetItemInfos(slotNum)
	local link = GetInventoryItemLink("player", slotNum);
	local _, _, color, enchant, name = string.find(link,"|c%x%x(%x*)|Hitem:%d+:(%d+):%d+:%d+:%d+:%d+:%-?%d+:%-?%d+:%d+:%d+|h%[([^%]]*)%]");
	
	return color, enchant, name;
end

----------------------
-- EventHandler
----------------------

function iGear:DurabilityChange()
	RepairCosts = 0;

	local smallest_durability = 100;
	local equipped, repcosts, durability, enchant;
	
	for _, s in ipairs(EquipSlots) do
		equipped, repcosts = GetItemEquippedAndCost(s[SLOT_ID]);
		
		if( not equipped ) then
			s[SLOT_IS_EQUIPPED] = false;
		else
			s[SLOT_IS_EQUIPPED] = true;
			s[SLOT_CAN_DURABILITY] = true;
			
			-- durability :)
			durability = GetItemDurability(s);
			if( not durability ) then
				s[SLOT_CAN_DURABILITY] = false;
			else
				s[SLOT_DURABILITY] = durability;
				
				if( durability < smallest_durability ) then
					smallest_durability = durability;
				end
				
				RepairCosts = RepairCosts + repcosts;
			end
			
			-- gems
			s[SLOT_GEMS_EMPTY] = NumMissingGems(s[SLOT_ID]);
			-- retrieve item color, name and enchant
			s[SLOT_COLOR], enchant, s[SLOT_ITEM_NAME] = GetItemInfos(s[SLOT_ID]);
			-- enchant
			s[SLOT_ENCHANT] = tonumber(enchant);
		end
	end
	
	self.Feed.text = ("|cff%s%d%%|r"):format(Crayon:GetThresholdHexColor(smallest_durability, 100), smallest_durability);
	
	local problems = self:GetNumProblems();
	if( problems and problems ~= 0 ) then
		self.Feed.text = ("%s |cffff0000%d!|r"):format(self.Feed.text, problems);
	end
end

-----------------------
-- UpdateTooltip
-----------------------

local function GetHintText(slot)
	local text = "";
	
	if( slot[SLOT_GEMS_EMPTY] ~= 0 ) then
		text = ("|cff%s%d %s|r"):format("ff0000", slot[SLOT_GEMS_EMPTY], slot[SLOT_GEMS_EMPTY] == 1 and L["Gem"] or L["Gems"]);
	end
	
	if( slot[SLOT_CAN_ENCHANT] and slot[SLOT_ENCHANT] == 0 ) then
		text = ("%s%s|cff00ffff%s|r"):format(text, text ~= "" and ", " or "", L["Enchant"]);
	end
	
	return text;
end

local function LineEnter(anchor, slotNum)
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
	GameTooltip:SetInventoryItem("player", slotNum);
	GameTooltip:Show();
end

local function LineLeave()
	GameTooltip:ClearLines();
	GameTooltip:Hide();
end

local TipAlign= {};
local TipHead = {};
local TipLine = {};
function iGear:UpdateTooltip()
	Tooltip:Clear();
	
	-- cols: durability, slot, icon, item, hints
	tclear(TipAlign);
	tclear(TipHead);
	
	-- we just show the durability column if there's something to repair (beautiful!)
	if( RepairCosts ~= 0 ) then
		table.insert(TipAlign, "RIGHT");
		table.insert(TipHead, "");
	end
	
	table.insert(TipAlign, "LEFT");
	table.insert(TipAlign, "RIGHT");
	table.insert(TipAlign, "LEFT");
	
	table.insert(TipHead, L["Slot"]);
	table.insert(TipHead, "");
	table.insert(TipHead, L["Item"]);
	
	local problems = self:GetNumProblems();
	if( problems > 0 ) then
		table.insert(TipAlign, "LEFT");
		table.insert(TipHead, L["Hints"]);
	end
	
	Tooltip:SetColumnLayout(#TipAlign, unpack(TipAlign));
	Tooltip:AddHeader(unpack(TipHead));
	
	local line;
	for _, s in ipairs(EquipSlots) do
		if( self:GetProblem(s, 1) ) then
			tclear(TipLine);
			
			if( RepairCosts ~= 0 ) then
				if( s[SLOT_CAN_DURABILITY] and s[SLOT_DURABILITY] ~= 100 ) then
					table.insert(TipLine, ("|cff%s%d%%|r"):format(Crayon:GetThresholdHexColor(s[SLOT_DURABILITY], 100), s[SLOT_DURABILITY]));
				else
					table.insert(TipLine, "");
				end
			end
			
			table.insert(TipLine, (COLOR_GOLD):format(L[s[SLOT_NAME]]));
			table.insert(TipLine, ("|T%s:14:14|t"):format(GetInventoryItemTexture("player", s[SLOT_ID])));
			table.insert(TipLine, ("|cff%s%s|r"):format(s[SLOT_COLOR], s[SLOT_ITEM_NAME] ~= "" and s[SLOT_ITEM_NAME] or UNKNOWN));
			
			if( problems > 0 ) then
				table.insert(TipLine, GetHintText(s));
			end
			
			line = Tooltip:AddLine(unpack(TipLine));
			Tooltip:SetLineScript(line, "OnEnter", LineEnter, s[SLOT_ID]);
			Tooltip:SetLineScript(line, "OnLeave", LineLeave);
		end
	end
	
	if( RepairCosts ~= 0 ) then
		Tooltip:AddLine(" ");
		Tooltip:AddLine("", (COLOR_GOLD):format(L["Repair Costs"]), "", GetMoneyString(RepairCosts));
	end
end